/*==============================================================================
 *  This file is part of the CHT_Value_Chart_Alert_COG program.
 *
 *  The CHT_Value_Chart_Alert_COG program is free software: you can redistribute
 *  it and/or modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CHT_Value_Chart_Alert_COG is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 *  more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CHT_Value_Chart_Alert_COG.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  FILE: CHT_Value_Chart_Alert_COG_v2.3.mq4
 *  VERSION: 2.3
 *  AUTHOR: Thierry Chappuis <tc77@pygnol.ch>
 *
 *  DESCRIPTION:
 *  This file is part of the CHT_Value_Chart_Alert_COG program, a MT4 custom
 *  indicator to give an alert when the value chart reach specified values. The
 *  value chart signals can be filtered using a center of gravity indicator such
 *  as MBFX (Belkhayate) or Regression.
 *
 *  LOG:
 *  - 20121104: version 0.1 and 1.0
 *  - 20121127: version 2.0b01 to 2.0b03
 *  - 20121128: redesign of the whole structure for MBFX+Regression integration
 *             version 2.0b04
 *  - 20121129: version 2.0 (release)
 *  - 20121203: version 2.1 (bug fix)
 *  - 20121203: version 2.2 (bug fix)
 *  - 20121204: version 2.3 (bug fix)
 *
 *  Copyright 2012, Thierry Chappuis
 *=============================================================================*/

#property copyright "Copyright 2012, Thierry Chappuis"

#property indicator_chart_window
#property indicator_buffers 7

/* User parameters ------------------------------------------------------------*/
extern string vc_Section =  "Value Chart Alert parameters ---------------------------------";
extern int vc_NumBars = 5;
extern int vc_NumLevels = 4;
extern double vc_level1 = 6;
extern double vc_level2 = -6;
extern double vc_level3 = 8;
extern double vc_level4 = -8;
extern double vc_level5 = 10;
extern double vc_level6 = -10;
// Time limit between two alerts ON of set to true
extern bool vc_alert_time_limit = true;

extern string cog_Section = "COG parameters (MBFX or Regression) --------------------------";
extern string cog_implementation = "MBFX"; // MBFX ou Regression
extern bool cog_alerts_on = false;
extern bool cog_filter_vc_alerts = true;
extern double cog_upper_level = 2.0;
extern double cog_lower_level = 2.0;
extern bool cog_levels_visible = true;

extern string mbfx_Section = "MBFX parameters ----------------------------------------------";
extern int mbfx_NumBars = 120;
extern int mbfx_RegressionDegree = 3;
extern double mbfx_Deviation = 1.61803399;

extern string reg_Section =  "Regression parameters ----------------------------------------";
extern int reg_NumBars = 200;
extern int reg_RegressionDegree = 4;
extern double reg_Deviation = 1.27;
extern double reg_Deviation2 = 1.618;
extern double reg_Deviation3 = 2.618;

extern string cog_ColorSection =  "Center of gravity colors -------------------------------------";
extern color centerline_color = Blue;
extern color upper_line1_color = DimGray;
extern color upper_line2_color = Red;
extern color upper_line3_color = Red;
extern color lower_line1_color = DimGray;
extern color lower_line2_color = LimeGreen;
extern color lower_line3_color = LimeGreen;

/* Global variables ------------------------------------------------- */
double vc_close;
double vc_levels[6];
bool vc_in[6];
datetime vc_in_time[6];

double cog_levels[8];
double cog_upper_limit;
double cog_lower_limit;
bool cog_in_upper;
bool cog_in_upper_time;
bool cog_in_lower;
bool cog_in_lower_time;

/* Buffers ---------------------------------------------------------- */
double centerline[];
double upper_gray[];
double upper_red_1[];
double upper_red_2[];
double lower_gray[];
double lower_green_1[];
double lower_green_2[];

/* Initialization function ----------------------------- */
int init()
{
    value_chart_alert_init();
    cog_alert_init();

    SetIndexBuffer(0, centerline);
    SetIndexBuffer(1, upper_gray);
    SetIndexBuffer(2, upper_red_1);
    SetIndexBuffer(3, upper_red_2);
    SetIndexBuffer(4, lower_gray);
    SetIndexBuffer(5, lower_green_1);
    SetIndexBuffer(6, lower_green_2);
    
    /* Init of the buffers */
    if (cog_levels_visible == true)
    {
        SetIndexStyle(0, DRAW_LINE, STYLE_DASH, 1, centerline_color);
        SetIndexStyle(1, DRAW_LINE, STYLE_SOLID, 1, upper_line1_color);
        SetIndexStyle(2, DRAW_LINE, STYLE_SOLID, 1, upper_line2_color);
        SetIndexStyle(3, DRAW_LINE, STYLE_SOLID, 1, upper_line3_color);
        SetIndexStyle(4, DRAW_LINE, STYLE_SOLID, 1, lower_line1_color);
        SetIndexStyle(5, DRAW_LINE, STYLE_SOLID, 1, lower_line2_color);
        SetIndexStyle(6, DRAW_LINE, STYLE_SOLID, 1, lower_line3_color);
    }
    else
    {
        SetIndexStyle(0, DRAW_NONE);
        SetIndexStyle(1, DRAW_NONE);
        SetIndexStyle(2, DRAW_NONE);
        SetIndexStyle(3, DRAW_NONE);
        SetIndexStyle(4, DRAW_NONE);
        SetIndexStyle(5, DRAW_NONE);
        SetIndexStyle(6, DRAW_NONE);
    }

    return(0);
}

/* Entry point ----------------------------------------- */
int start()
{
    int err = 0;

    if (cog_levels_visible == true || cog_alerts_on == true || cog_filter_vc_alerts == true)
    {

        if (cog_implementation == "MBFX")
        {
            err = mbfx_indicator();
        }
        else
        {
            if (cog_implementation == "Regression")
            {
                err = reg_indicator();
            }
            else
            {
                err = -1;
            }
        }
    }

    if (err == 0)
    {
        cog_computes_alert_limits();
        cog_alerts(); 

        value_chart_compute();
        value_chart_alerts();
    }

    return(err);
}

/* Value chart functions -----------------------------------------------------*/
void value_chart_alert_init()
{
    /* VC alert system initialization */
    vc_levels[0] = vc_level1;
    vc_levels[1] = vc_level2;
    vc_levels[2] = vc_level3;
    vc_levels[3] = vc_level4;
    vc_levels[4] = vc_level5;
    vc_levels[5] = vc_level6;

    for (int i = 0; i < vc_NumLevels; i++)
    {
        vc_in[i] = false;
        vc_in_time[i] = 0;
    }
}

void value_chart_compute()
{
    double sum;
    double floatingAxis;
    double volatilityUnit;

    // Computation of the floating axis value
    sum = 0;
    for (int i = 0; i < vc_NumBars; i++)
    {
        sum += (High[i] + Low[i]) / 2.0;
    }

    floatingAxis = sum / vc_NumBars;

    // Computation of the volatility unit value
    sum = 0;
    for (i = 0; i < vc_NumBars; i++)
    {
        sum += High[i] - Low[i];
    }

    volatilityUnit = 0.2 * (sum / vc_NumBars);

    // Computation of the relative close value
    vc_close = (Close[0] - floatingAxis) / volatilityUnit;
}

void value_chart_alerts()
{
    bool alert_lock = false;

    // Alert system
    for (int i = 0; i < vc_NumLevels; i++)
    {
        double level = vc_levels[i];

        if (level > 0)
        {
            if (vc_close >= level && vc_in[i] == false)
            {
                if (vc_alert_time_limit == true)
                {
                    alert_lock = (vc_in_time[i] == Time[0]);
                }
                else
                {
                    alert_lock = false;
                }

                if (cog_in_upper == true || cog_filter_vc_alerts == false)
                {
                    if (alert_lock == false)
                    {
                        Alert(Symbol(), " (", Period(), " min) signal: Value chart above ", level, "!");
                        vc_in_time[i] = Time[0];
                    }
      
                    vc_in[i] = true;
                }
            }

            if (vc_close < level && vc_in[i] == true)
            {
                vc_in[i] = false;
            }
        }

        if (level < 0)
        {
            if (vc_close <= level && vc_in[i] == false)
            {

                if (vc_alert_time_limit == true)
                {
                    alert_lock = (vc_in_time[i] == Time[0]);
                }
                else
                {
                    alert_lock = false;
                }

                if (cog_in_lower == true || cog_filter_vc_alerts == false)
                {
                    if (alert_lock == false)
                    {
                        Alert(Symbol(), " (", Period(), " min) signal: Value chart below ", level, "!");
                        vc_in_time[i] = Time[0];
                    }
                    
                    vc_in[i] = true;
                }
            }

            if (vc_close > level && vc_in[i] == true)
            {
                vc_in[i] = false;
            }
        }
    }
}

/* COG functions ------------------------------------------------------------ */

/**
 * Initializes global variables for use with the COG alert subsystem.
 */
void cog_alert_init()
{
    /* COG alert system initialization */
    cog_in_upper = false;
    cog_in_lower = false;

    cog_in_upper_time = 0;
    cog_in_lower_time = 0;
}

/**
 * Computes and levels where a COG alert must be triggered according to the user
 * parameters
 */
void cog_computes_alert_limits()
{
    if (cog_filter_vc_alerts || cog_alerts_on)
    {
        int upper_level;
        int lower_level;
        double upper_rel_position;
        double lower_rel_position;

        cog_upper_level = MathAbs(cog_upper_level);
        cog_lower_level = MathAbs(cog_lower_level);

        if (cog_upper_level < 3.0)
        {
            upper_level = MathFloor(cog_upper_level) + 0.1; // 0, 1 or 2
            upper_rel_position = cog_upper_level - MathFloor(cog_upper_level);
        }
        else
        {
            upper_level = 2;
            upper_rel_position = cog_upper_level - 2.0;

        }

        if (cog_lower_level < 3.0)
        {
            lower_level = MathFloor(cog_lower_level) + 0.1; // 0, 1 or 2
            lower_rel_position = cog_lower_level - MathAbs(cog_lower_level);
        }
        else
        {
            lower_level = 2;
            lower_rel_position = cog_lower_level - 2.0;
        }

        
        cog_upper_limit = cog_levels[upper_level] + upper_rel_position * (cog_levels[upper_level+1] - cog_levels[upper_level]);
        lower_level = lower_level + 4;
        cog_lower_limit = cog_levels[lower_level] - lower_rel_position * (cog_levels[lower_level] - cog_levels[lower_level+1]);
    }
}

/**
 * Triggers an alert when the center of gravity reach an upper or lower limit.
 */
void cog_alerts()
{
    bool alert_lock;

    if (cog_filter_vc_alerts == true || cog_alerts_on == true)
    {
        if (Close[0] > cog_upper_limit && cog_in_upper == false)
        {
            cog_in_upper = true;

            if (vc_alert_time_limit == true)
            {
                alert_lock = (cog_in_upper_time == Time[0]);
            }
            else
            {
                alert_lock = false;
            }

            if (cog_alerts_on == true && alert_lock == false)
            {
                Alert(Symbol(), " (", Period(), " min) COG signal: price is above upper line ", DoubleToStr(cog_upper_level,1), "!");
            }
            cog_in_upper_time = Time[0];

        }

        if (Close[0] <= cog_upper_limit && cog_in_upper == true)
        {
            cog_in_upper = false;
        }

        if (Close[0] < cog_lower_limit && cog_in_lower == false)
        {
            cog_in_lower = true;

            if (vc_alert_time_limit == true)
            {
                alert_lock = (cog_in_lower_time == Time[0]);
            }
            else
            {
                alert_lock = false;
            }

            if (cog_alerts_on == true && alert_lock == false)
            {
                Alert(Symbol(), " (", Period(), " min) MBFX signal: price is is below lower line ", DoubleToStr(cog_lower_level,1), "!");
            }
            cog_in_lower_time = Time[0];
        }

        if (Close[0] >= cog_lower_limit && cog_in_lower == true)
        {
            cog_in_lower = false;
        }
    }
}


/* MBFX functions ------------------------------------------------------------ */

/**
 * Computes the levels (centerline + 6 levels) for the MBFX center of gravity implementation.
 */
int mbfx_indicator()
{
    int err = 0;
    int i, j, k;

    int n;
    double sum;
    double std_dev;

    double a[20][20];
    double sx[20];
    double b[20];
    double x[20];

    n = mbfx_RegressionDegree + 1;
    sx[1] = mbfx_NumBars + 1;

    // ---- sx ---------------------------------------------------------------
    for (j = 1; j <= n * 2 - 2; j++)
    {
        sum = 0;
        for (i = 0; i <= mbfx_NumBars; i++)
        {
            sum += MathPow(i, j);
        }
        sx[j + 1] = sum;
    }

    // ---- vector b ---------------------------------------------------------
    for (j = 1; j <= n; j++)
    {
        sum = 0;
        for (i = 0; i <= mbfx_NumBars; i++)
        {
            if (j == 1)
            {
                sum += (High[i] + Low[i]) / 2.0;
            }
            else
            {
                sum += (High[i] + Low[i]) / 2.0 * MathPow(i, j - 1);
            }
        }
        b[j] = sum;
    }

    // ---- matrix a ---------------------------------------------------------
    for (i = 1; i <= n; i++)
    {
        for (j = 1; j <= n; j++)
        {
            k = i + j - 1;
            a[i][j] = sx[k];
        }
    }

    // ---- Gauss ------------------------------------------------------------
    GaussElimination(a, b, n, x);

    // ---- SQ ---------------------------------------------------------------
    for (i = 0; i <= mbfx_NumBars; i++)
    {
        sum = 0;
        for (k = 1; k <= mbfx_RegressionDegree; k++)
        {
            sum += (x[k+1]) * MathPow(i, k);
        }
        centerline[i] = x[1] + sum;
    }

    std_dev = iStdDev(NULL, 0, mbfx_NumBars, 0, MODE_SMA, PRICE_HIGH, 0) * mbfx_Deviation;

    for (i = 0; i <= mbfx_NumBars; i++)
    {
        upper_red_2[i] = centerline[i] + std_dev;
        upper_red_1[i] = centerline[i] + (upper_red_2[i] - centerline[i]) / 1.382;
        upper_gray[i] = centerline[i] + (upper_red_1[i] - centerline[i]) / 1.618;
        lower_green_2[i] = centerline[i] - std_dev;
        lower_green_1[i] = centerline[i] - (centerline[i] - lower_green_2[i]) / 1.382;
        lower_gray[i] = centerline[i] - (centerline[i] - lower_green_1[i]) / 1.618;
    }

    /* We return the levels */
    cog_levels[0] = centerline[0];
    cog_levels[1] = upper_gray[0];
    cog_levels[2] = upper_red_1[0];
    cog_levels[3] = upper_red_2[0];
    cog_levels[4] = centerline[0];
    cog_levels[5] = lower_gray[0];
    cog_levels[6] = lower_green_1[0];
    cog_levels[7] = lower_green_2[0];

    // Drawing settings
    SetIndexDrawBegin(0, Bars - mbfx_NumBars - 1);
    SetIndexDrawBegin(1, Bars - mbfx_NumBars - 1);
    SetIndexDrawBegin(2, Bars - mbfx_NumBars - 1);
    SetIndexDrawBegin(3, Bars - mbfx_NumBars - 1);
    SetIndexDrawBegin(4, Bars - mbfx_NumBars - 1);
    SetIndexDrawBegin(5, Bars - mbfx_NumBars - 1);
    SetIndexDrawBegin(6, Bars - mbfx_NumBars - 1);

    return (err);
}


/* Regression functions ---------------------------------------------------- */

int reg_indicator()
{
    int err = 0;
    int i, j, k;

    int n;
    double sum;

    double dev = 0.0;
    double dev2;
    double dev3;

    double a[20][20];
    double sx[20];
    double b[20];
    double x[20];

    n = reg_RegressionDegree + 1;
    sx[1] = reg_NumBars + 1;

    // ---- sx ---------------------------------------------------------------
    for (j = 1; j <= n * 2 - 2; j++)
    {
        sum = 0;
        for (i = 0; i <= reg_NumBars; i++)
        {
            sum += MathPow(i, j);
        }
        sx[j + 1] = sum;
    }

    // ---- vector b ---------------------------------------------------------
    for (j = 1; j <= n; j++)
    {
        sum = 0;
        for (i = 0; i <= reg_NumBars; i++)
        {
            if (j == 1)
            {
                sum += Close[i];
            }
            else
            {
                sum += Close[i] * MathPow(i, j - 1);
            }
        }
        b[j] = sum;
    }

    // ---- matrix a ---------------------------------------------------------
    for (i = 1; i <= n; i++)
    {
        for (j = 1; j <= n; j++)
        {
            k = i + j - 1;
            a[i][j] = sx[k];
        }
    }

    // ---- Gauss ------------------------------------------------------------
    GaussElimination(a, b, n, x);

    // ---- SQ ---------------------------------------------------------------
    for (i = 0; i <= reg_NumBars; i++)
    {
        sum = 0;
        for (k = 1; k <= reg_RegressionDegree; k++)
        {
            sum += (x[k+1]) * MathPow(i, k);
        }
        centerline[i] = x[1] + sum;
        dev += MathPow(Close[i] - centerline[i], 2);
    }

    dev3 = reg_Deviation3 * MathSqrt(dev/(reg_NumBars+1));
    dev2 = reg_Deviation2 * MathSqrt(dev/(reg_NumBars+1));
    dev = reg_Deviation * MathSqrt(dev/(reg_NumBars+1));


    for (i = 0; i <= reg_NumBars; i++)
    {
        upper_red_2[i] = centerline[i] + dev3;
        upper_red_1[i] = centerline[i] + dev2;
        upper_gray[i] = centerline[i] + dev;
        lower_green_2[i] = centerline[i] - dev3;
        lower_green_1[i] = centerline[i] - dev2;
        lower_gray[i] = centerline[i] - dev;
    }
    
    /* We return the levels */
    cog_levels[0] = centerline[0];
    cog_levels[1] = upper_gray[0];
    cog_levels[2] = upper_red_1[0];
    cog_levels[3] = upper_red_2[0];
    cog_levels[4] = centerline[0];
    cog_levels[5] = lower_gray[0];
    cog_levels[6] = lower_green_1[0];
    cog_levels[7] = lower_green_2[0];

    // Drawing settings
    SetIndexDrawBegin(0, Bars - reg_NumBars - 1);
    SetIndexDrawBegin(1, Bars - reg_NumBars - 1);
    SetIndexDrawBegin(2, Bars - reg_NumBars - 1);
    SetIndexDrawBegin(3, Bars - reg_NumBars - 1);
    SetIndexDrawBegin(4, Bars - reg_NumBars - 1);
    SetIndexDrawBegin(5, Bars - reg_NumBars - 1);
    SetIndexDrawBegin(6, Bars - reg_NumBars - 1);

    return (err);
}

/*Utility functions -------------------------------------------------------- */

/**
* This function solves a system of max_linear equations using 
* the Gauss elimination method.
*
* a: square matrix of size (n+1)x(n+1) containing the coefficients of
*    the equations. max_line/column 0 of the matrix are not considered so
*    that the matrix effective size is nxn.
* b: vector of size n containing the results of each equation composing
*    the system.
* n: size of the n x n matrix containing the coefficients.
* x (OUT): vector of size n receiving the solutions of the system.
*
* return: the fonction returns the value 1 if an error occurent. The value
*         o is returned if no error occured.
*/
int GaussElimination(double & a[][], double & b[], int n, double & x[])
{
    int err = 0;

    int i, j, k, l; // Matrix indices

    double max_value;
    int max_line;
    double quotient;
    double temp;
    double sum;

    /* First, transfort matrix a in its row echelon form */
    for(k = 1; k <= n-1; k++)
    {
        max_line = 0;
        max_value = 0;

        /* looking for the maximum coefficient in each column */
        for(i = k; i <= n; i++)
        {
            if (MathAbs(a[i,k]) > max_value)
            {
                max_value = MathAbs(a[i,k]);
                max_line = i;
            }
        }

        if (max_line == 0)
        {
            // Error
            err = 1;
            return(err);
        }

        if (max_line != k)
        {
            for(j = 1; j <= n; j++)
            {
                /* max_lines k and max_line of the matrix are swaped */
                temp = a[k,j];
                a[k,j] = a[max_line,j];
                a[max_line,j] = temp;
            }

            /* elements k and max_line of the vector are swaped */
            temp = b[k];
            b[k] = b[max_line];
            b[max_line] = temp;
        }

        for(i = k+1; i <= n; i++)
        {
            quotient = a[i,k] / a[k,k];

            for(j=1; j <= n; j++)
            {
                if (j==k)
                {
                    a[i,j] = 0;
                }
                else
                {
                    a[i,j] = a[i,j] - quotient * a[k,j];
                }
            }

            b[i] = b[i] - quotient * b[k];
        }
    }

    /* Computes the solutions of the system of linar equations
       from the row echelon matrix b */
    x[n] = b[n] / a[n,n];

    for(i = n - 1; i >= 1; i--)
    {
        sum = 0;

        for(j = 1; j <= n - i ; j++)
        {
            sum += a[i,i+j] * x[i+j];
            x[i] = (1/a[i,i]) * (b[i] - sum);
        }
    }

    return(err);
}